function result = cvSelect(Data, param, varargin)

% Runs a cross-validation to select the best parameters. In each fold, some
% positive examples are labeled as negative, then baggingSVM is run, and we
% record the predicted scores of all unlabeled. The AUC is the criterion to
% compare the performance between the different values of the parameters.
% If an additionnal argument NbPos is present, the function will
% run an 'inverse' cross-validation to evaluate the performance of
% the method when used with NbPos positive examples. The positive set is
% divided in sets of NbPos examples. In each fold, one of these sets is
% used for training while the remainder is used for testing. Then similarly
% to the preceding case, a parameter is selected and the predicted scores
% of all unlabeled examples are recorded.
% Alternatively, if another argument 'Nrepeats' is present, Nrepeats sets of
% NbPos positive examples are randomly chosen to serve for training.
%
% INPUT :
%  - Data : a structure containing the features of the training
%  examples, their split into set of positive and unlabeled examples and
%  various parameters, as initialized by initialize()
%
%  - param : a structure containing a grid of values for each parameter to
%  be optimized. It must contain at least 'C', a grid for the SVM regularization
%  parameter. Then, for bagging, the structure can contain 'K', a vector of
%  values for the size of the subsample used as negative train set and
%  optionally, 'r', a grid for the unbalance ratio between positive and
%  negative examples (r=C+/C-). 
%  K=NaN means negative training set size is set to positive training set size.
%  r=0 means that standard ratio=N-/N+ is used.
%
% OPTIONAL ARGUMENTS :
%  - NbPos : an integer indicating how many positive examples should be
%  used for learning.
%  - Nrepeats : an integer which makes the function perform a bootstrap
%  validation. Nrepeats sets of NbPos positive examples are randomly
%  chosen for learning.
%
% OUTPUT :
%   - result :  a structure containing
%       o bestParam : the selected parameters, C, K and r
%       o score : the predicted scores for the unlabeled examples
%
% USE :
% param=struct('C', [1 10 100],'r',0,'K',NaN)
% cvresult = cvSelect(Data, param) or
% cvresult = cvSelect(Data, param, 'NbPos', 10) or
% cvresult = cvSelect(Data, param, 'NbPos', 10, 'Nrepeats', 20)

NbPos=NaN;
Nrepeats=NaN;

% Parse optional arguments
l=1;
while (l<length(varargin))
    eval([varargin{l},' = varargin{',num2str(l+1),'};']);
    l = l+2;
end

posSplit = split(Data, 'NbPos',NbPos, 'Nrepeats', Nrepeats);

sData = Data;
sData.featureTest = '';

Auc = NaN(length(param.C), length(param.K),length(param.r));

% Loop over the parameters among which to select, perform a cross
% validation for each of them
for l = 1:length(param.C)
    for m=1:length(param.K)
        for j = 1:length(param.r)
            sData.method.param.C=param.C(l);
            sData.method.param.K=param.K(m);
            sData.method.param.r=param.r(j);
            
            % preallocating
            cvresult = cell(1, length(posSplit));
            % initialization
            for k=1:length(posSplit)
                cvresult{k} = struct('label', [], 'score', []);
            end
            
            for ifold = 1:length(posSplit)
                % run inference method with only the training examples of
                % the fold as positives
                
                % assign labels: -1 for true negative, 0 for true positive used as
                % positive training examples, 1 for true positive used as negative
                % training examples
                if ~isnan(NbPos) && isnan(Nrepeats)
                    sData.posSet = posSplit{ifold}.test;
                    label = -ones(Data.nexample,1);
                    label(posSplit{ifold}.test) = 0;
                    label(posSplit{ifold}.train) = 1;
                    
                else
                    sData.posSet = posSplit{ifold}.train;
                    label = -ones(Data.nexample,1);
                    label(posSplit{ifold}.train) = 0;
                    label(posSplit{ifold}.test) = 1;
                    
                end
                score = predict(sData);
                % record the results of the fold
                cvresult{ifold}.label = label;
                switch lower(Data.method.name)
                    case {'baggingsvm' ,'baggingmkl'}
                        cvresult{ifold}.score = score.score;
                    otherwise
                        cvresult{ifold}.score = score;
                end
            end
            % compute performance of each parameter
            res = analyzeTEMP(cvresult);
            Auc(l,m,j) = res.meanAuc;
        end
    end
end

% select 'best' parameter (which maximizes AUC)
[a b] = max(Auc(:));
[x y z] = ind2sub(size(Auc), b);
bestParam = struct('C', param.C(x), 'K',param.K(y),'r', param.r(z));
 
result = struct('bestParam', bestParam, 'score', struct(),'AUC',Auc,'AUP',Aup);
if ~strcmp(sData.method.name, '1class')
    % rescaling of C so that lambda*N*C is constant
    bestParam.C=bestParam.C*(Data.nsplitCvInt-1)/Data.nsplitCvInt;
end
result.bestParam.C=bestParam.C;
Data.method.param.C = bestParam.C;
Data.method.param.r = bestParam.r;
Data.method.param.K = bestParam.K;

% predict with these parameters
result.score = predict(Data);
