function Data = initialize(featureFile,posSet,method,varargin)

% function Data = initialize(featureFile,posSet,varargin)
%
% Read data for local PU learning
%
% INPUT
%  - featureFile : generally speaking, this argument is used to enter the
%  training data. It can be of three types :
%       1) name of a file containing numeric values for the train example 
%       features. The successive rows are the features of the successive
%       ezamples. All rows should have the same number of attributes.
%       2) it can contain a pre-computed Gram matrix but then, the
%       'kernelType'field of parameter 'method', should be set to
%       'precomputed'
%       3) featureFile can be given as a cell array of strings 
%       specifying 'mat' filenames, where different kernels are stored, in 
%       case one wants to perform MKL (1class or bagging).
%   See also additional argument 'fileType' for more details on how to input
%   the features
%  - posSet : a index vector of the positive examples, the rest of the
%  examples is considered to be the unlabeled set.
%  - method : a structure regarding the inference method with the following
%    fields, should be built with function 'fmethod'
%       > name :
%       [baggingSVM|MKL1class|baggingMKL]
%       > param : a parameter structure with fields:
%         - param.C : the 'C' regularization parameter of the SVM. Be
%         careful that for one-class SVM, this corresponds in reality to
%         parameter nu
%           (default: 0.1)
%         - param.kernelType : type of kernel used,
%         ['linear'|'rbf'|'poly'|'precomputed']
%         If the kernel is precomputed, the featureFile contains the Gram
%         matrix rather than the feature matrix         
%           (default : gaussian 'rbf')
%         - param.kernelParam : the kernel parameters
%            o if a gaussian kernel is chosen, this is the 'sigma' parameter 
%              (default: a heuristic is used to adapt it to the data, only 
%               valid when data are normalized)
%            o otherwise it should be the degree and constant coefficient for a
%              polynomial kernel
%              (default = -1, which does not make any sense)
%         - param.K : a parameter of the bagging method. If specified, the
%         size of the bootstrap subsamples.
%              (default = NaN)
%         - param.Kmult : same as above, except that the size of the
%         bootstrap subsamples will be chosen as a multiple of the size of
%         the positive set (Kmult*NP)
%              (default = 1)
%
% ADDITIONAL ARGUMENTS
%  Several other arguments can be input, by specifying the argument name
%  followed by its value:
%  - fileType : ['mat'|'txt'] This indicates the type of feature files
%  provided. In the first case, a mat file will be loaded, whereas in the
%  second case, a tab-delimited file containing numeric values will be
%  read. (default : 'txt') 
%  Anytime a mat file is provided, the variable containing the feature 
%  matrix should be named 'feature' and the variable containing the kernel 
%  should be named 'Kernel'.
%  - nsplitCvOut : number of folds for the outer loop of double cross-validation 
%    (default: 3)
%  - nsplitCvIn : number of folds for the inner loop of double cross-validation 
%    (default: 3)
%  - normalizeFeatures : normalize (1) or not (0) the example features to
%    zero mean and unit standard deviation
%    (default: 1).
%  - defaultPositiveScore : the score of known positive examples when they
%    are predicted
%    (default: 100)
%  - rocCurveLength : the length of the ROC and PR curves 
%    (default : 1000)
%  
% OUTPUT:
%  a structure variable with the following fields:
%      o nexample : the number of train examples
%      o nfeature : the number of features
%      o feature : it can be a nexample x nfeature matrix of train
%      features or a nexample x nexample kernel matrix if 'kernelType' is
%      'precomputed'.
%      In the latter case, field 'nfeature' is not in the structure.
%      o nexampleTest : the number of test examples
%      o featureTest : a nexampleTest x p matrix of test features
%      o posSet : list of positive examples
%      o kernelType : type of kernel used
%      o method : a structure
%          > name : the name of the inference method
%          > param : a parameter structure
%             - C : the SVM C parameter
%             - kernelParam : the kernel parameters
%             - kernelType : the kernel type
%             - nsplitPredict : the number of folds for prediction
%      o nsplitCvIn : the number of folds for inner cross-validation
%      o nsplitCvOut : the number of folds for outer cross-validation
%      o normalizeFeatures : whether or not the example features are
%      normalized
%      o defaultPositiveScore : the score of known positive edges 
%      o rocCurveLength : length of ROC and PR curves
%
% EXAMPLE:
% d = initialize('expression.txt', posSet, method, 'fileType', 'txt')
%
% SEE ALSO: function 'fmethod'

Data.method = method;

% Default values for additional arguments
Data.nsplitCvOut = 3;
Data.nsplitCvInt = 3;
Data.normalizeFeatures = 1;
Data.defaultPositiveScore = 100;
Data.rocCurveLength = 1000;
Data.fileType ='mat';

% Parse optional arguments
l=1;
while (l<length(varargin))
    Data.(varargin{l}) = varargin{l+1};
    l = l+2;
end

if iscell(featureFile)
    % featureFile is a cell which contains the mat files where the different
    % kernels for MKL optimization are stored, in a variable named 'Kernel'
    if ~strcmp(Data.method.name,'MKL1class') && ~strcmp(Data.method.name,'baggingMKL')
        error('Argument ''featureFile'' must be character string')
    end
    if isempty(Data.nexample)
        error('For MKL, the number of examples should be provided by the user.')
    end
    Data.feature=featureFile;
    
elseif ischar(featureFile)
    % featureFile contains either a feature matrix or a kernel matrix
    if ~strcmp( Data.method.param.kernelType, 'precomputed')
        % Read the example features
        if strcmp(Data.fileType, 'txt')
            Data.feature = dlmread(featureFile);
        else
            load(featureFile, 'feature');
            Data.feature = feature;
        end
        if strcmp(class(Data.feature),'single')
            Data.feature=double(Data.feature);
        end
        [Data.nexample,Data.nfeature] = size(Data.feature);
        if (Data.normalizeFeatures)
            Data.feature = (Data.feature - mean(Data.feature,2)*ones(1,Data.nfeature)) ./ (std(Data.feature')'*ones(1,Data.nfeature)); %#ok<UDIM>
        end
        Data.feature = sparse(Data.feature);
        
        % default parameter of rbf kernel
        if (strcmp(Data.method.param.kernelType , 'rbf') && Data.method.param.kernelParam <0)
            Data.method.param.kernelParam  = sqrt(2*Data.nfeature)/4;
            % a heuristic that does not make sense if data are not normalized
        end
    else
        % read the precomputed kernel
        if strcmp(Data.fileType, 'txt')
            Data.feature = dlmread(featureFile);
        else
            load(featureFile,'Kernel')
            Data.feature=Kernel;
        end
        if strcmp(class(Data.feature),'single')
            Data.feature=double(Data.feature);
        end
        Data.nexample = size(Data.feature,1);
    end
else
   error('Wrong argument ''featureFile'', unknown type of data') 
end

% Add positive examples
Data.posSet=posSet;



