function result = predict_baggingmkl(Data)

% USAGE
% result = predict_baggingmkl(Data)
%
% Predicts scores for unlabeled examples in Data using bagging MKL
%
% REQUIRED:
%  - source directory of SeDuMi should be on the path
%
% INPUT :
%  - Data : a structure containing the features of the training
%  examples, their split into set of positive and unlabeled examples and
%  various parameters, as initialized by initialize()
%
% OUTPUT:
%  a vector of predicted scores for the unlabeled examples from Data
%  object, mean score and bootstrap subsamples

NbScore = Data.method.param.NbScore;

labels = -ones(Data.nexample,1);
labels(Data.posSet)=1;

% size of bootstrap samples. If not specified, take a multiple of the size
% of the positive set
K = Data.method.param.K;
if isnan(K)
    K=ceil(Data.method.param.Kmult*length(Data.posSet));
end

result = struct('scoreBrut',[],'score',zeros(Data.nexample,1),'Record',struct([]));

% define positive and unlabeled set
trainpos = find(labels>0);
npos = size(trainpos,1);
Utrain = find(labels<0);

Record=struct('trainneg',[],'diffset',[]);
% count the number of times an element falls into a bootstrap subsample
count = zeros(Data.nexample,1);
count(trainpos) = NbScore;

% libsvm bugs when there is no positive example
if npos==0
    result.mean(:)=-1;
else
    % draw randomly subsamples of size K, make sure that each example is
    % chosen at least NbScore times
    i=1;
    while any(count<NbScore)
        Record(i).trainneg = randsample(Utrain, K,Data.method.param.replace);
        Record(i).diff = setdiff(Utrain, Record(i).trainneg);
        count(Record(i).diff) = count(Record(i).diff)+1;
        i=i+1;
    end
    
    scoretemp = NaN(Data.nexample, i-1);
    
    % extract the list of Gram matrices for each data type and each
    % bootstrap subsample
    
    % initialize
    Klist = cell(1,length(Record));
    Klist2 = cell(1,length(Record));
    for i=1:length(Record)
        Klist{i} = cell(1,length(Data.feature));
        Klist2{i} = cell(1,length(Data.feature));
    end
    
    Kernel=[];
    % loop on the kernels
    for k=1:length(Data.feature)
        load(Data.feature{k},'Kernel')
        for i=1:length(Record)
            alltrain = [trainpos ;Record(i).trainneg];
            Klist{i}{k} = double(Kernel(alltrain,alltrain));
            Klist2{i}{k} = double(Kernel(alltrain,:));
        end
        clear Kernel
    end
    
    % train the MKL-L2-SVM for each bootstrap sample
    alpha = NaN(length(Record),npos+K);
    theta = NaN(length(Record),length(Data.feature));
    for i=1:length(Record)
        alltrain = [trainpos ;Record(i).trainneg];
        [alpha(i,:), theta(i,:), obj,b] =  socp_weighted_classicalSVM_sedumi_L2_binary(Klist{i},labels(alltrain),Data.method.param.C);
    end
    
    % compute predicted scores
    for k=1:length(Data.feature)
        for i=1:length(Record)
            alltrain = [trainpos ;Record(i).trainneg];
            v=theta(i,k)*(labels(alltrain).*alpha(i,:)')'*Klist2{i}{k}(:,Record(i).diff);
            if k>1
                scoretemp(Record(i).diff,i)=scoretemp(Record(i).diff,i)+v';
            else
                scoretemp(Record(i).diff,i)=v;
            end
        end
    end

result.scorebrut = scoretemp;
result.score=nanmean(scoretemp,2);
result.Record=Record;
end