
function result = predict_baggingsvm(Data)

% result = predict_baggingsvm(Data)
%
% The goal is to infer associations between a disease and genes. The 
% function predicts scores for unlabeled examples in Data using bagging SVM.
%
% REQUIRED:
%  - source directory of libsvm should be on the path
%
% INPUT :
%  - Data : a structure containing the features of the training
%  examples, their split into set of positive and unlabeled examples and
%  various parameters, as initialized by initialize()
%
% OUTPUT:
%  a vector of predicted scores for the unlabeled examples from Data
%  object, mean score and bootstrap subsamples.

NbScore = Data.method.param.NbScore;

labels = -ones(Data.nexample,1);
labels(Data.posSet)=1;

% svm parameters
switch Data.method.param.kernelType
    case 'rbf'
        % don't forget to convert sigma to gamma for the Gaussian kernel
        gamma = 1/(2*Data.method.param.kernelParam(1)^2);
        param = strcat(['-t 2 -g ',num2str(gamma),' -c ',num2str(Data.method.param.C)]);
    case 'poly'
        param = strcat(['-t 1 -g 1 -t',num2str(Data.method.param.kernelParam(1)),'-r ',Data.method.param.kernelParam(2),' -c ',num2str(Data.method.param.C)]);
    case 'linear'
        param = strcat(['-t 0 -c ',num2str(Data.method.param.C)]);
    case 'precomputed'
        param = strcat(['-t 4 -c ',num2str(Data.method.param.C)]);
    otherwise
        error('Kernel type unknown');
end

% size of bootstrap samples. If not specified, take a multiple of the size
% of the positive set
K = Data.method.param.K;
if isnan(K)
    K=ceil(Data.method.param.Kmult*length(Data.posSet));
end

result = struct('scoreBrut',[],'score',zeros(Data.nexample,1),'Record',struct([]));

% define positive and unlabeled set
trainpos = find(labels>0);
npos = size(trainpos,1);
Utrain = find(labels<0);

switch Data.method.param.r
    case 0
        % balanced C with theoretical weigths (default behavior)
        param=[param, ' -w1 ',num2str(K/(npos+K)), ' -w-1 ',num2str(1-K/(npos+K))];
    otherwise
        % balanced C with weights given by the user
        w1 = Data.method.param.r/(1+Data.method.param.r);
        w0 = 1/(1+Data.method.param.r);
        param=[param, ' -w1 ',num2str(w1), ' -w-1 ',num2str(w0)];
end

Record=struct('trainneg',[],'diff',[]);
% count the number of times an element falls into a bootstrap subsample
count = zeros(Data.nexample,1);
count(trainpos) = NbScore;

% libsvm bugs when there is no positive example
if npos==0
    result.mean(:)=-1;    
else
    i=1;
    while any(count<NbScore)
         Record(i).trainneg = randsample(Utrain, K,Data.method.param.replace);
         Record(i).diff = setdiff(Utrain, Record(i).trainneg);
         count(Record(i).diff) = count(Record(i).diff)+1;
        i=i+1;
    end
    
    scoretemp = NaN(length(Record),Data.nexample);
    % train a SVM for all boostrap samples
    for i=1:length(Record)
        alltrain = [trainpos ;Record(i).trainneg];     

        if ~strcmp(Data.method.param.kernelType, 'precomputed')
            model = svmtrain(labels(alltrain), Data.feature(alltrain,:),param);
            [p,a,v] = svmpredict(labels(Record(i).diff),Data.feature(Record(i).diff,:),model);
        else         
         model=svmtrain(labels(alltrain),[(1:length(alltrain))',Data.feature(alltrain,alltrain)],param);
         [p,a,v] = svmpredict(labels(Record(i).diff),[(1:length(Record(i).diff))',Data.feature(Record(i).diff,alltrain)],model);
        end
        
        scoretemp(i,Record(i).diff) = v*labels(alltrain(1));
        result.score(Record(i).diff) = result.score(Record(i).diff)+v*labels(alltrain(1));

    end
          
    result.scoreBrut = scoretemp;
    result.score=result.score./(count-1);
    result.Record=Record;
end

